require 'rails/generators'
require 'awesome_print'

module Appboost
  class ChefSoloGenerator < Rails::Generators::Base
    desc "Generates chef"

    source_root File.join(File.dirname(__FILE__), '../../../templates')

    argument :host, type: :string, desc: 'Example: google.com'

    class_option :database,
      type: :string,
      default: 'postgresql',
      desc: 'mysql or postgresql',
      aliases: '-d'

    class_option :ssh_pub_key,
      type: :string,
      default: "#{ENV['HOME']}/.ssh/id_rsa.pub"

    class_option :app_user,
      type: :string,
      default: 'app'

    class_option :vagrant_box,
      type: :string,
      default: "opscode_ubuntu-12.04-i386_chef-11.4.4"

    class_option :vagrant_box_url,
      type: :string,
      default: "https://opscode-vm.s3.amazonaws.com/vagrant/opscode_ubuntu-12.04-i386_chef-11.4.4.box"

    class_option :db_password,
      type: :string,
      default: 'password'

    class_option :ruby,
      type: :string,
      default: '2.0.0-p0'

    def copy_files
      template "Cheffile.erb",    "chef/Cheffile"
      template "Vagrantfile.erb", "chef/Vagrantfile"
      template "node.json.erb",   "chef/nodes/#{host}.json"
      template "user.json.erb",   "chef/data_bags/users/#{options[:app_user]}.json"

      copy_file "chef.gitignore", "chef/.gitignore"
    end

    private

    def mysql?
      options[:database] == 'mysql'
    end

    def postgresql?
      options[:database] == 'postgresql'
    end

    def postgresql_config
      {
        config: {
          listen_addresses: "*",
          port: "5432"
        },
        pg_hba: [
          {
            type:   "local",
            db:     "postgres",
            user:   "postgres",
            addr:   nil,
            method: "trust"
          },
          {
            type:   "host",
            db:     "all",
            user:   "all",
            addr:   "0.0.0.0/0",
            method: "md5"
          },
          {
            type:   "host",
            db:     "all",
            user:   "all",
            addr:   "::1/0",
            method: "md5"
          }
        ],
        password: {
          postgres: options[:db_password]
        }
      }
    end

    def mysql_config
      {
        :server_root_password   => options[:db_password],
        :server_repl_password   => options[:db_password],
        :server_debian_password => options[:db_password],
        :service_name           => "mysql",
        :basedir                => "/usr",
        :data_dir               => "/var/lib/mysql",
        :root_group             => "root",
        :mysqladmin_bin         => "/usr/bin/mysqladmin",
        :mysql_bin              => "/usr/bin/mysql",
        :conf_dir               => "/etc/mysql",
        :confd_dir              => "/etc/mysql/conf.d",
        :socket                 => "/var/run/mysqld/mysqld.sock",
        :pid_file               => "/var/run/mysqld/mysqld.pid",
        :grants_path            => "/etc/mysql/grants.sql"
      }
    end

    def authorization_config
      {
        sudo: {
          users: [options[:app_user]],
          passwordless: true
        }
      }
    end    

    def common_config
      cfg = {
        nginx: {
          dir: "/etc/nginx",
          log_dir: "/var/log/nginx",
          binary: "/usr/sbin/nginx",
          user: "www-data",
          pid: "/var/run/nginx.pid",
          worker_connections: "1024"
        },
        git: {
          prefix: "/usr/local"
        },
      }
      cfg[:mysql] = mysql_config if mysql?
      cfg[:postgresql] = postgresql_config if postgresql?
      cfg
    end

    def config
      common_config.merge({
        authorization: authorization_config,
        rbenv: rbenv_config(options[:app_user])
      })
    end

    def config_json
      config.merge({
        run_list: run_list
      })
    end

    def vagrant_config
      # common_config.merge({
      #   rbenv: rbenv_config('vagrant')
      # })
      config
    end

    def run_list
      [
        "apt",
        "chef-solo-search",
        "locale",
        "users::sysadmins",
        "sudo",
        "runit",
        mysql? ? "mysql::server" : nil,
        postgresql? ? "postgresql::server" : nil,
        "imagemagick",
        "ruby_build",
        "rbenv::user",
        "nginx::repo",
        "nginx",
        "git"
      ].compact
    end
    
    def app_name
      Dir.pwd.split('/').last
    end

    def user_json
      {
        id:        options[:app_user],
        comment:   "Application User",
        ssh_keys:  [File.read(options[:ssh_pub_key])],
        groups:    ["sysadmin", "sudo", "staff"],
        shell:     "/bin/bash"
      }
    end


    def rbenv_config(user) 
      {
        user_installs: [{
          user: user,
          rubies: [ options[:ruby] ],
          global: options[:ruby],
          environment: { CFLAGS: "-march=native -O2 -pipe" },
          gems: {
            options[:ruby] => [{name: "bundler", version: "1.3.5"}]
          }
        }]
      }
    end
  end
end